# Generated by Django 3.1.3 on 2020-11-20 13:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('drf_auth_service', '0002_auto_20201109_1254'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ssouser',
            name='service',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='service_users', to='drf_auth_service.service'),
        ),
    ]
